﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace WpfApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void AddItemButton_Click(object sender, RoutedEventArgs e)
        {
            (this.DataContext as TimelineViewModel).AddNewItem();
            (this.DataContext as TimelineViewModel).ZoomOutAccordingToData();
        }
    }

    public class TimelineViewModel : ViewModelBase
    {
        public ObservableCollection<TimelineItem> TimelineItems { get; private set; }
 
        private DateTime periodStart;

        public DateTime PeriodStart
        {
            get
            {
                return this.periodStart;
            }
            set
            {
                if (this.periodStart != value)
                {
                    this.periodStart = value;
                    this.OnPropertyChanged(() => this.PeriodStart);
                }
            }
        }
 
        private DateTime periodEnd;

        public DateTime PeriodEnd
        {
            get
            {
                return this.periodEnd;
            }
            set
            {
                if (this.periodEnd != value)
                {
                    this.periodEnd = value;
                    this.OnPropertyChanged(() => this.PeriodEnd);
                }
            }
        }
 
        private DateTime visiblePeriodStart;

        public DateTime VisiblePeriodStart
        {
            get
            {
                return this.visiblePeriodStart;
            }
            set
            {
                if (this.visiblePeriodStart != value)
                {
                    this.visiblePeriodStart = value;
                    this.OnPropertyChanged(() => this.VisiblePeriodStart);
                }
            }
        }
 
        private DateTime visiblePeriodEnd;

        public DateTime VisiblePeriodEnd
        {
            get
            {
                return this.visiblePeriodEnd;
            }
            set
            {
                if (this.visiblePeriodEnd != value)
                {
                    this.visiblePeriodEnd = value;
                    this.OnPropertyChanged(() => this.VisiblePeriodEnd);
                }
            }
        }
 
        public TimelineViewModel()
        {
            this.periodStart = new DateTime(2013, 1, 1);
            this.TimelineItems = new ObservableCollection<TimelineItem>();
            GenerateTestData();
            ZoomOutAccordingToData();
        }
 
        private void GenerateTestData()
        {
            for (int i = 0; i < 40; i++)
            {
                this.AddNewItem();
            }
        }
 
        public void AddNewItem()
        {
            int i = this.TimelineItems.Count;
 
            this.TimelineItems.Add(new TimelineItem
            {
                StartDate = this.periodStart.AddDays(i),
                Duration = TimeSpan.FromHours(6.0)
            });
        }

        public void ZoomOutAccordingToData()
        {
            DateTime min = this.TimelineItems.Min(item => item.StartDate);
            DateTime max = this.TimelineItems.Max(item => item.StartDate.Add(item.Duration));

            this.PeriodStart = min;
            this.PeriodEnd = max;
            this.VisiblePeriodStart = min;
            this.VisiblePeriodEnd = max;
        }
    }
 
    public class TimelineItem
    {
        public DateTime StartDate { get; set; }

        public TimeSpan Duration { get; set; }
    }
}
